""" 
Humid Air Properties
http://www.coolprop.org/fluid_properties/HumidAir.html#table-of-inputs-outputs-to-hapropssi

"""


class EC_Humid_Air(object):
    
    def __init__(self, T=None ,P=None):
        pass
